import sys,time,random
import zlib, base64, getpass, atexit

def compress(data, level=9):
	''' return compressed, ASCII-encoded string '''
	return base64.encodestring(zlib.compress(data,9))

def decompress(data):
	''' return uncompressed data '''
	return zlib.decompress(base64.decodestring(data))

typing_speed = 100 #wpm
def slow_type(t):
	for l in t:
		sys.stdout.write(l)
		sys.stdout.flush()
		time.sleep(random.random()*10.0/typing_speed)
	print ''

def display(name):
    user = getpass.getuser()
    if compress(user) in faces:
        nl.notify(decompress(faces[compress(user)][name]))

def set_goodbye():
    atexit.register(display,'goodbye')

faces = {
    'eNrLLcqvyk3MAwAMOAMF\n': {
        'greeting': 'eNplUDEKwzAM3P2KSxclBctjIS/o0B9ErSdDh0ChdNTjK9kxJalsZEln684K6MahR4mAvLMOpYSE\ng4mDXDG16FrW9YXPs7yLo+qoSkqXih7MUWM7QeB7T5uCt8XZlt2CpYqxMjoXiLNX/DTndWJQaAKF\nIuUYI2PCFHjPOjMF/fuFVrbqeBsItUjRtZteebiaHGluE3FCbTjZF6y1Sd0KEPE3AhlNRx+g9egv\nWorbgOGXYcl3qy1ZwxdvpUwA\n',
        'goodbye': 'eNp1T8sKgzAQvOcrxl5iIK73iIfe+hGChJo+IK3QSqGXfHs3aqoUO5DsZGd2lgDbKPmIjb4m+qNp\nyZesFk1rattIWpbKqFDSjIzuwIgvSZAhxey9R9ffXSYmI3Iu6rtFGZ2oSLMEqgtGTVQEUCVnw8G+\nHCxu9nw9Wo+Hew7oTxguDp19Z9MGsJsnjBm54bgdVv/LTeyOtFLrxRFNg8lOJX61OWepwAc6pzyM\n'
    },
    'eNpLSi/KLy4GAAjDApE=\n': {
        'greeting': 'eNrzSM3JyVcoyUgtStVRKM4s0gMANzUF9Q==\n',
        'goodbye': 'eNpzz89PSapM1QMADcIC+A==\n'
    }
}



from gini.matching import best_match_from_list

class Concept(object):
    def __init__(self,name,example_list=None):
        #: Arbitrary label
        self.name = name
        #: List of example strings
        self.examples = []
        if example_list:
            self.examples = example_list
        #: Method to run if this is used as an action
        self.action = None
        #: Any information to attach to this object
        self.meta = None
        #: [return value] Name of the matching item
        self.match = None
        #: [return value] This concept is currently being used in the negative sense (i.e., "not ____")
        self.negative = False
        #: [return value] The type of match this was found in (e.g., ``exact`` or ``fuzzy``)
        self.match_type = None
        #: [return value] The degree of match (when using fuzzy matching)
        self.match_amount = None
    
    def match_string(self,string,fuzzy=90,fname_match=True,fuzzy_fragment=None,guess=False):
        m = best_match_from_list(string,self.examples,fuzzy,fname_match,fuzzy_fragment,guess)
        if m:
            self.match = m[0]
            self.match_type = m[2]
            self.match_amount = m[3]
            return self
        return None

class Bottle(object):
    '''Container for semantic information (and magical creatures)'''
    def __init__(self,vocab=[]):
        #: list of :class:`Concepts` to match with
        self.vocab = vocab
        #: matching options
        self.fname_match = True
        self.fuzzy = 90
        self.fuzzy_fragment = None
        self.guess = False
        #: Allow the use of "not" and a "-" before a term to negate the term
        self.negative = True
    
    def match_word(self,string):
        multipliers = {'exact':10**5,'fname':10**4,'fuzzy':10**2,'fuzzy_fragment':1}
        matches = []
        for concept in self.vocab:
            match = concept.match_string(string,self.fuzzy,self.fname_match,self.fuzzy_fragment,self.guess)
            if match:
                matches.append((multipliers[match.match_type]*(match.match_amount if match.match_amount else 1),match))
        if len(matches)>0:
            return sorted(matches,reverse=True)[0][1]
        return None
    
    def matches_from(self,string):
        if isinstance(string,list):
            items = string
        else:
            items = string.split()
        item_dict = {}
        not_next = False
        for item in items:
            if self.negative:
                if item=='not':
                    not_next = True
                    continue
                if item[0]=='-':
                    not_next = True
                    item = item[1:]
            concept = self.match_word(item)
            if concept:
                if not_next:
                    concept.negative = True
                if concept.name not in item_dict:
                    item_dict[concept.name] = []
                item_dict[concept.name].append(concept)
            else:
                if 'other' not in item_dict:
                    item_dict['other'] = []
                item_dict['other'].append(item)
            not_next = False
        return item_dict
    
    def process(self,string):
        '''Searches the string (or list of strings) for an action word (a :class:`Concept` that has and ``action`` attached
        to it), then calls the appropriate function with a dictionary of the identified words (according to ``vocab``).
        
        For example, if you have your ``vocab`` set like::
        
            print_concept = Concept('print', ['print','show','list','blah'])
            print_concept.action = print_me
            bottle.vocab = [
                print_concept,
                Concept('say', ['say','speak','sprechen']),
                Concept('money', ['money','moolah','cash','denaros']),
                Concept('self', ['me','user'])
            ]
        
        Then this command::
        
            bottle.process('Show me the money')
        
        Would result in this function being called::
        
            print_me({'money': [Concept('money')], 'self': [Concept('me')], 'other': ['the']})
        
        '''
        item_dict = self.matches_from(string)
        for item in item_dict:
            if 'action' in dir(item_dict[item][0]) and item_dict[item][0].action:
                item_dict[item][0].action(item_dict)


from fuzzywuzzy import fuzz
import fnmatch,glob

def matches_from_list(item,options,fuzzy=90,fname_match=True,fuzzy_fragment=None,guess=False):
    '''Returns the members of ``options`` that best matches ``item``. Will prioritize
    exact matches, then filename-style matching, then fuzzy matching. Returns a tuple of item,
    index, match type, and fuzziness (if applicable)
    
         :item:             string to match
         :options:          list of examples to test against
         :fuzzy:            integer (out of 100) describing how close to match string
         :fname_match:      use filename globbing to match files?
         :fuzzy_fragment:   if not ``None``, will accept substring matches of
                            at least ``fuzzy_fragment`` fuzziness
         :guess:            if ``True``, shortcut for setting ``fuzzy`` and ``min_fragment``
                            to very lenient options
    '''
    matches = []
    
    if guess:
        fuzzy = min(fuzzy,80)
        fuzzy_fragment = min(fuzzy_fragment,70)
    
    option_not_in = lambda item,match_list: all([x[0]!=item for x in match_list])
    
    # Exact matches
    if item in options:
        matches += [(options[i],i,'exact',None) for i in xrange(len(options)) if options[i].lower()==item.lower()]
    
    # Filename-style matches
    if fname_match:
        matches += [(x,options.index(x),'fname',None) for x in fnmatch.filter(options,item) if option_not_in(x,matches)]
    
    # Fuzzy matches
    if fuzzy:
        sub_matches = []
        for i in xrange(len(options)):
            r = fuzz.ratio(item.lower(),options[i].lower())
            if r>=fuzzy and option_not_in(options[i],matches):
                sub_matches.append((r,i))
        matches += [(options[x[1]],x[1],'fuzzy',x[0]) for x in sorted(sub_matches)]
    
    # Fragment matches
    if fuzzy_fragment:
        sub_matches = []
        for i in xrange(len(options)):
            r = fuzz.partial_ratio(item.lower(),options[i].lower())
            if r>=fuzzy_fragment and option_not_in(options[i],matches):
                sub_matches.append((r,i))
        matches += [(options[x[1]],x[1],'fuzzy_fragment',x[0]) for x in sorted(sub_matches)]
    
    return matches

def best_match_from_list(item,options,fuzzy=90,fname_match=True,fuzzy_fragment=None,guess=False):
    '''Returns the best match from :meth:`matches_from_list` or ``None`` if no good matches'''
    matches = matches_from_list(item,options,fuzzy,fname_match,fuzzy_fragment,guess)
    if len(matches)>0:
        return matches[0]
    return None

def best_item_from_list(item,options,fuzzy=90,fname_match=True,fuzzy_fragment=None,guess=False):
    '''Returns just the best item, or ``None``'''
    match = best_match_from_list(item,options,fuzzy,fname_match,fuzzy_fragment,guess)
    if match:
        return match[0]
    return None


from distutils.core import setup

version = '0.6.1'

setup(
  name = 'gini',
  packages = ['gini'], # this must be the same as the name above
  version = version,
  description = 'The un-intelligent Python AI library',
  long_description = '''Gini stands for "Gini is not intelligent." The library is for making tools that probably fall under the general field of AI, for people who don't care about AI.

Put another way, these are functions to help my scripts be polite, and act flexibly. I don't want my scripts to gain sentience. I just want them to interact with humans smoother.

''',
  author = 'Bill Gross',
  author_email = 'bill.gross@me.com',
  url = 'https://github.com/azraq27/gini',
  download_url = 'https://github.com/azraq27/gini/tarball/' + version,
  keywords = ['AI'],
  classifiers = [
      'Topic :: Scientific/Engineering',
      'Intended Audience :: Science/Research',
      'Development Status :: 3 - Alpha'
  ],
  install_requires=[
      'fuzzywuzzy',
      'python-Levenshtein'
  ]
)
